/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.common;

import java.beans.ConstructorProperties;
import java.io.Serializable;
import java.util.Objects;
import java.util.regex.Pattern;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;

/**
 * A universally unique identifier (UUID).
 * The string representation is as defined in RFC 4122.
 * The canonical representation uses lower case characters.
 * <p>
 * The following is an example UUID:
 * <pre>
 *     f81d4fae-7dec-11d0-a765-00a0c91e6bf6
 * </pre>
 */
public class Uuid implements Serializable {
    private static final long serialVersionUID = -6700962668564346825L;

    // package private for unit test access
    static final String E_BAD_FORMAT =
            "Supplied value \"%s\" does not match required pattern \"%s\"";
    static final String UUID_PATTERN = "^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-" +
            "[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$";

    private static final Pattern RE_UUID = Pattern.compile(UUID_PATTERN);


    private final String value;

    /**
     * Creates a new instance from a given value.
     *
     * @param value value
     */
    @ConstructorProperties("value")
    public Uuid(String value) {
        checkNotNull(value, "Supplied value may not be null");
        checkArgument(RE_UUID.matcher(value).matches(),
                E_BAD_FORMAT, value, UUID_PATTERN);
        this.value = value;
    }

    /**
     * Creates a copy from the given universally unique identifier.
     *
     * @param source universally unique id
     */
    public Uuid(Uuid source) {
        this.value = source.value;
    }

    /**
     * Returns the value of this instance.
     *
     * @return the value
     */
    public String getValue() {
        return value;
    }

    @Override
    public int hashCode() {
        return value.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Uuid other = (Uuid) obj;
        return Objects.equals(value, other.value);
    }
}

