/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import org.onosproject.nemo.model.common.UserId;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * Representation of the end transaction input builder.
 */
public class EndTransactionInputBuilder {

    private Map<Class<? extends EndTransactionInput>, EndTransactionInput>
            augmentation = new HashMap<>();

    private UserId userId;

    /**
     * Creates an end transaction input builder.
     */
    public EndTransactionInputBuilder() {
    }

    /**
     * Creates an end transaction input builder from
     * the end transaction input.
     *
     * @param base EndTransactionInput object
     */
    public EndTransactionInputBuilder(EndTransactionInput base) {
        userId = base.getUserId();
        if (base instanceof EndTransactionInputImpl) {
            EndTransactionInputImpl impl = (EndTransactionInputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the user ID for the builder to use.
     *
     * @param id user's ID
     * @return self
     */
    public EndTransactionInputBuilder userId(UserId id) {
        userId = id;
        return this;
    }

    /**
     * Adds augmentation by a class type and an end transaction input.
     * If the augmentation is null, remove the augmentation type key
     * from the hash map.
     *
     * @param augmentationType class extends EndTransactionInput
     * @param augmentation EndTransactionInput object
     * @return self
     */
    public EndTransactionInputBuilder addAugmentation(
            Class<? extends EndTransactionInput> augmentationType,
            EndTransactionInput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation a class type.
     *
     * @param augmentationType class that extends EndTransactionInput
     * @return self
     */
    public EndTransactionInputBuilder removeAugmentation(
            Class<? extends EndTransactionInput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds an end transaction input builder object.
     *
     * @return an end transaction input builder object
     */
    public EndTransactionInput build() {
        return new EndTransactionInputImpl(this);
    }

    /**
     * Representation of the implement of EndTransactionInput.
     */
    private static final class EndTransactionInputImpl
            implements EndTransactionInput {

        private final UserId userId;

        private final Map<Class<? extends EndTransactionInput>,
                EndTransactionInput> augmentation;

        private EndTransactionInputImpl(EndTransactionInputBuilder base) {
            userId = base.userId;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public UserId getUserId() {
            return userId;
        }

        @Override
        public int hashCode() {
            return Objects.hash(userId, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            EndTransactionInputImpl other = (EndTransactionInputImpl) obj;
            return Objects.equals(userId, other.getUserId()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("userId", userId)
                    .add("augmentation", augmentation).toString();
        }
    }
}