/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.google.common.base.MoreObjects.toStringHelper;

/**
 * Representation of language style request output builder.
 */
public class LanguageStyleNemoRequestOutputBuilder
        extends DefaultOutputBuilderBase {

    private Map<Class<? extends LanguageStyleNemoRequestOutput>,
            LanguageStyleNemoRequestOutput> augmentation = new HashMap<>();

    /**
     * Constructs a language style request output builder object.
     */
    public LanguageStyleNemoRequestOutputBuilder() {
    }

    /**
     * Constructs a language style request output builder object.
     *
     * @param rpc common RPC result
     */
    public LanguageStyleNemoRequestOutputBuilder(CommonRpcResult rpc) {
        resultCode = rpc.getResultCode();
        message = rpc.getMessage();
    }

    /**
     * Constructs a language style request output object.
     *
     * @param base language style request output object
     */
    public LanguageStyleNemoRequestOutputBuilder(
            LanguageStyleNemoRequestOutput base) {
        message = base.getMessage();
        resultCode = base.getResultCode();
        if (base instanceof LanguageStyleNemoRequestOutputImpl) {
            LanguageStyleNemoRequestOutputImpl impl =
                    (LanguageStyleNemoRequestOutputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Adds augmentation by a class type and a language style request output.
     * If the augmentation is null, remove the augmentation type key
     * from the hash map.
     *
     * @param augmentationType type that extends LanguageStyleNemoRequestOutput
     * @param augmentation language style request output object
     * @return self
     */
    public LanguageStyleNemoRequestOutputBuilder addAugmentation(
            Class<? extends LanguageStyleNemoRequestOutput> augmentationType,
            LanguageStyleNemoRequestOutput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType type that extends LanguageStyleNemoRequestOutput
     * @return self
     */
    public LanguageStyleNemoRequestOutputBuilder removeAugmentation(
            Class<? extends LanguageStyleNemoRequestOutput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a language style request output object.
     *
     * @return a language style request output object
     */
    public LanguageStyleNemoRequestOutput build() {
        return new LanguageStyleNemoRequestOutputImpl(this);
    }

    /**
     * Representation of the implement of LanguageStyleNemoRequestOutput.
     */
    private static final class LanguageStyleNemoRequestOutputImpl
            extends DefaultOutputImplBase
            implements LanguageStyleNemoRequestOutput {

        private final Map<Class<? extends LanguageStyleNemoRequestOutput>,
                LanguageStyleNemoRequestOutput> augmentation;

        private LanguageStyleNemoRequestOutputImpl(
                LanguageStyleNemoRequestOutputBuilder base) {
            message = base.message;
            resultCode = base.resultCode;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public int hashCode() {
            return Objects.hash(message, resultCode, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            LanguageStyleNemoRequestOutputImpl other =
                    (LanguageStyleNemoRequestOutputImpl) obj;
            return Objects.equals(message, other.getMessage()) &&
                    Objects.equals(resultCode, other.getResultCode()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("message", message)
                    .add("resultCode", resultCode)
                    .add("augmentation", augmentation).toString();
        }
    }
}