/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.operation.action.definitions.ActionDefinition;

/**
 * Representation of building an action definitions.
 */
public class ActionDefinitionsBuilder {

    private Map<Class<? extends ActionDefinitions>, ActionDefinitions>
            augmentation = new HashMap<>();
    private List<ActionDefinition> actionDefinition;

    /**
     * Creates a default action definitions builder.
     */
    public ActionDefinitionsBuilder() {
    }

    /**
     * Creates an action definitions builder from a given action definitions.
     *
     * @param base the action definitions
     */
    public ActionDefinitionsBuilder(ActionDefinitions base) {
        actionDefinition = base.getActionDefinitions();
        if (base instanceof ActionDefinitionsImpl) {
            ActionDefinitionsImpl impl = (ActionDefinitionsImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the action definition to be used by the builder.
     *
     * @param definition action definition
     * @return self
     */
    public ActionDefinitionsBuilder actionDefinitions(
            List<ActionDefinition> definition) {
        actionDefinition = definition;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public ActionDefinitionsBuilder addAugmentation(
            Class<? extends ActionDefinitions> augmentationType,
            ActionDefinitions augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation = new HashMap<>();
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public ActionDefinitionsBuilder removeAugmentation(
            Class<? extends ActionDefinitions> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds an action definitions from this builder instance.
     *
     * @return action definitions
     */
    public ActionDefinitions build() {
        return new ActionDefinitionsImpl(this);
    }

    /**
     * The implementation of action definitions.
     */
    private static final class ActionDefinitionsImpl
            implements ActionDefinitions {

        private final List<ActionDefinition> actionDefinition;
        private final Map<Class<? extends ActionDefinitions>,
                ActionDefinitions> augmentation;

        private ActionDefinitionsImpl(ActionDefinitionsBuilder base) {
            actionDefinition = base.actionDefinition;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public List<ActionDefinition> getActionDefinitions() {
            return actionDefinition;
        }

        @Override
        public int hashCode() {
            return Objects.hash(actionDefinition, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ActionDefinitions other = (ActionDefinitions) obj;
            if (!Objects.equals(actionDefinition,
                    other.getActionDefinitions())) {
                return false;
            }
            ActionDefinitionsImpl otherImpl = (ActionDefinitionsImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("actionDefinition", actionDefinition)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}