/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.operation.condition.parameter.definitions.ConditionParameterDefinition;

/**
 * Representation of building a condition parameter definitions.
 */
public class ConditionParameterDefinitionsBuilder {

    private Map<Class<? extends ConditionParameterDefinitions>,
            ConditionParameterDefinitions> augmentation = new HashMap<>();
    private List<ConditionParameterDefinition> conditionParameterDefinition;

    /**
     * Creates a default condition parameter definitions builder.
     */
    public ConditionParameterDefinitionsBuilder() {
    }

    /**
     * Creates a action definitions builder from a given
     * condition parameter definitions.
     *
     * @param base the condition parameter definitions
     * @throws ClassCastException if the given class can not cast
     */
    public ConditionParameterDefinitionsBuilder(
            ConditionParameterDefinitions base) {
        conditionParameterDefinition = base.getDefinitions();
        if (base instanceof ConditionParameterDefinitions) {
            ConditionParameterDefinitionsImpl impl =
                    (ConditionParameterDefinitionsImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the condition parameter definitions to be used by the builder.
     *
     * @param definition condition parameter definitions
     * @return self
     */
    public ConditionParameterDefinitionsBuilder conditionParameterDefinitions(
                    List<ConditionParameterDefinition> definition) {
        conditionParameterDefinition = definition;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public ConditionParameterDefinitionsBuilder addAugmentation(
            Class<? extends ConditionParameterDefinitions> augmentationType,
            ConditionParameterDefinitions augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public ConditionParameterDefinitionsBuilder removeAugmentation(
            Class<? extends ConditionParameterDefinitions> augmentationType) {
        this.augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a condition parameter definitions from this builder instance.
     *
     * @return condition parameter definitions
     */
    public ConditionParameterDefinitions build() {
        return new ConditionParameterDefinitionsImpl(this);
    }

    /**
     * The implementation of condition parameter definitions.
     */
    private static final class ConditionParameterDefinitionsImpl implements
            ConditionParameterDefinitions {

        private final List<ConditionParameterDefinition>
                conditionParameterDefinition;
        private final Map<Class<? extends ConditionParameterDefinitions>,
                ConditionParameterDefinitions> augmentation;

        private ConditionParameterDefinitionsImpl(
                ConditionParameterDefinitionsBuilder base) {
            conditionParameterDefinition = base.conditionParameterDefinition;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public List<ConditionParameterDefinition> getDefinitions() {
            return conditionParameterDefinition;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ConditionParameterDefinitions other =
                    (ConditionParameterDefinitions) obj;
            if (!Objects.equals(conditionParameterDefinition,
                    other.getDefinitions())) {
                return false;
            }
            ConditionParameterDefinitionsImpl otherImpl =
                    (ConditionParameterDefinitionsImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public int hashCode() {
            return Objects.hash(conditionParameterDefinition, augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("conditionParameterDefinition",
                            conditionParameterDefinition)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}