/*
 * Copyright 2017-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.action.instance;

import org.onosproject.nemo.model.operation.action.instance
         .parameter.values.IntValue;
import org.onosproject.nemo.model.operation.action.instance
         .parameter.values.RangeValue;
import org.onosproject.nemo.model.operation.action.instance
         .parameter.values.StringValue;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * The description of builder for parameter value objects.
 */
public class ParameterValuesBuilder {

    private final Map<Class<? extends ParameterValues>,
            ParameterValues> augmentation = new HashMap<>();
    private List<IntValue> intValue;
    private RangeValue rangeValue;
    private List<StringValue> stringValue;

    /**
     * Creates an instance by default.
     */
    public ParameterValuesBuilder() {
    }

    /**
     * Creates an parameter value builder from a given parameter value.
     *
     * @param base parameter value
     */
    public ParameterValuesBuilder(ParameterValues base) {
        this.intValue = base.getIntValues();
        this.rangeValue = base.getRangeValues();
        this.stringValue = base.getStringValues();
    }

    /**
     * Sets the list of integer value to be used by the builder.
     *
     * @param value list of integer value
     * @return self
     */
    public ParameterValuesBuilder intValue(List<IntValue> value) {
        this.intValue = value;
        return this;
    }

    /**
     * Sets the range value to be used by the builder.
     *
     * @param value range value
     * @return self
     */
    public ParameterValuesBuilder rangeValue(RangeValue value) {
        this.rangeValue = value;
        return this;
    }

    /**
     * Sets the string value to be used by the builder.
     *
     * @param value string value
     * @return self
     */
    public ParameterValuesBuilder stringValue(List<StringValue> value) {
        this.stringValue = value;
        return this;
    }

    /**
     * Adds the augmentation to be used by the builder.
     * Remove the augmentation, if the parameter value is null.
     *
     * @param augmentationType augmentation type
     * @param augmentation the augmentation to be used
     * @return self
     */
    public ParameterValuesBuilder addAugmentation(
            Class<? extends ParameterValues> augmentationType,
            ParameterValues augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation to be used by parameter value builder.
     *
     * @param augmentationType remove augmentation
     * @return a parameter value builder
     */
    public ParameterValuesBuilder removeAugmentation(
            Class<? extends ParameterValues> augmentationType) {
        this.augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a new parameter value instance.
     *
     * @return the parameter value instance
     */
    public ParameterValues build() {
        return new ParameterValuesImpl(this);
    }

    /**
     * The implementation of parameter value to create an instance.
     */
    private static final class ParameterValuesImpl implements ParameterValues {

        private final List<IntValue> intValue;
        private final RangeValue rangeValue;
        private final List<StringValue> stringValue;
        private Map<Class<? extends ParameterValues>,
                ParameterValues> augmentation;

        private ParameterValuesImpl(ParameterValuesBuilder base) {
            this.intValue = base.intValue;
            this.rangeValue = base.rangeValue;
            this.stringValue = base.stringValue;
            this.augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public List<IntValue> getIntValues() {
            return intValue;
        }

        @Override
        public RangeValue getRangeValues() {
            return rangeValue;
        }

        @Override
        public List<StringValue> getStringValues() {
            return stringValue;
        }

        @Override
        public int hashCode() {
            return Objects.hash(intValue, rangeValue,
                    stringValue, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            ParameterValues other = (ParameterValues) obj;
            ParameterValuesImpl otherImpl = (ParameterValuesImpl) obj;
            return Objects.equals(intValue, other.getIntValues()) &&
                    Objects.equals(rangeValue, other.getRangeValues()) &&
                    Objects.equals(stringValue, other.getStringValues()) &&
                    Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("intValue", intValue)
                    .add("rangeValue", rangeValue)
                    .add("stringValue", stringValue)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }

}
