/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.common;

import com.google.common.testing.EqualsTester;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.onosproject.nemo.model.common.Name.E_BAD_FORMAT;
import static org.onosproject.nemo.model.common.Name.E_BAD_LENGTH;
import static org.onosproject.nemo.model.common.Name.MAX_LENGTH;
import static org.onosproject.nemo.model.common.Name.MIN_LENGTH;
import static org.onosproject.nemo.model.common.Name.NAME_PATTERN;

/**
 * Unit tests for {@link Name}.
 */
public class NameTest extends AbstractCommonTest {


    private static final String NAME = "SomeName";
    private static final String OTHER = "OtherName";

    private static final String[] BAD_STRINGS = {
            "%..",
            "123"
    };
    private static final String[] GOOD_STRINGS = {
            "test",
            "test1",
            "test123"
    };

    private static final int TOO_FEW = MIN_LENGTH - 1;
    private static final int TOO_MANY = MAX_LENGTH + 1;

    private static final String EXP_TOO_SMALL = String.format(E_BAD_LENGTH, TOO_FEW);
    private static final String EXP_TOO_LONG = String.format(E_BAD_LENGTH, TOO_MANY);

    private Name name;
    private Name copy;
    private Name diff;

    @Test
    public void fromString() {
        name = new Name(NAME);
        assertEquals("name not set", NAME, name.getValue());
    }

    @Test
    public void copyConstructor() {
        name = new Name(NAME);
        copy = new Name(name);
        new EqualsTester()
                .addEqualityGroup(name, copy)
                .testEquals();
    }

    @Test(expected = NullPointerException.class)
    public void constructWithNullString() {
        new Name((String) null);
    }

    @Test(expected = NullPointerException.class)
    public void copyConstructWithNull() {
        new Name((Name) null);
    }

    @Test
    public void nameTooSmall() {
        try {
            name = new Name("");
            fail(E_NOEX);
        } catch (IllegalArgumentException e) {
            assertEquals(E_WREX, EXP_TOO_SMALL, e.getMessage());
        }
    }

    @Test
    public void nameTooLong() {
        try {
            StringBuilder temp = new StringBuilder();
            for (int i = 0; i < (MAX_LENGTH + 1); i++) {
                temp.append("a");
            }
            name = new Name(temp.toString());
            fail(E_NOEX);
        } catch (IllegalArgumentException e) {
            assertEquals(E_WREX, EXP_TOO_LONG, e.getMessage());
        }
    }

    @Test
    public void nameNotMatch() {
        for (String s : BAD_STRINGS) {
            try {
                new Name(s);
                fail(E_NOEX + COLON + s);
            } catch (IllegalArgumentException e) {
                assertEquals(E_WREX, String.format(E_BAD_FORMAT, s,
                        NAME_PATTERN), e.getMessage());
            }
        }
    }

    @Test
    public void nameMatch() {
        for (String s : GOOD_STRINGS) {
            try {
                new Name(s);
            } catch (IllegalArgumentException e) {
                fail(E_UNEX + COLON + s);
            }
        }
    }

    @Test
    public void equality() {
        name = new Name(NAME);
        copy = new Name(NAME);
        diff = new Name(OTHER);
        new EqualsTester()
                .addEqualityGroup(name, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }
}